{% extends "layout.html" %}

{% block content %}
<div class="container h-100">
    <div class="row h-100 justify-content-center align-items-center">
        <img src="https://api.iconify.design/flowbite:paper-plane-solid.svg?color=%231520A6" width="200" height="200">
        <h1 class="text-center">Kamihi</h1>
        <p class="text-center">Administration interface</p>
        <h3 class="text-center">Everything is running smoothly</h3>
    </div>
</div>
{% endblock %}
