"""
Telegram module for Kamihi.

This module provides the communication with the Telegram API

License:
    MIT

"""

from .client import TelegramClient
from .send import reply, send

__all__ = ["TelegramClient", "reply", "send"]
